# -*- coding: utf-8 -*-
import argparse
import numpy as np
import sys
import time

import gnssrefl.gps as g


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("orbit", help="orbit name (gps,gnss,gps+glo, or specific e.g. jax) ", type=str)
    parser.add_argument("year", help="year", type=int)
    parser.add_argument("month", help="month (or day of year)", type=int)
    parser.add_argument("day", help="day (zero if you use day of year earlier)", type=int)
    parser.add_argument("-doy_end", default=None, help="doy end for multi-day download ", type=str)
    parser.add_argument("-hour", default=None, help="optional request for ultrarapid hour", type=int)

    args = parser.parse_args().__dict__

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def download_orbits(orbit: str, year: int, month: int, day: int, doy_end: int = None, hour: int = 0 ):
    """
    command line interface for download_orbits. If day is zero, then it is assumed that 
    the month record is day or year

    Examples
    --------
    download_orbits nav 2020 50 0
        downloads broadcast orbits for day of year 50 in the year 2020
    download_orbits nav 2020 1 1 
        downloads broadcast orbits for January 1, 2020
    download_orbits gnss 2023 1 1 
        multi-GNSS orbits from GFZ
    download_orbits rapid 2023 1 1 
        rapid multi-GNSS orbits from GFZ
    download_orbits rapid 2023 1 0 -doy_end 10
        rapid multi-GNSS orbits from GFZ for days of year 1 thru 10 in 2023

    Parameters
    ----------

    orbit : string
        value options:

            gps (default) : uses GPS broadcast orbit

            gps+glo : will use JAXA orbits which have GPS and Glonass (usually available in 48 hours)

            gnss : will use GFZ orbits, which is multi-GNSS (available in 3-4 days). but taken from CDDIS archive

            nav : GPS broadcast, adequate for reflectometry. Searches various places

            nav-sopac : GPS broadcast file from SOPAC, adequate for reflectometry. 

            nav-esa : GPS broadcast file from ESA, adequate for reflectometry. 

            nav-cddis : GPS broadcast file from CDDIS, very slow to download

            igs : IGS precise, GPS only

            igr : IGS rapid, GPS only

            jax : JAXA, GPS + Glonass, within a few days, missing block III GPS satellites

            gbm : GFZ Potsdam, multi-GNSS, not rapid

            grg : French group, GPS, Galileo and Glonass, not rapid

            esa : ESA, multi-GNSS

            gfr : GFZ rapid, GPS, Galileo and Glonass, since May 17 2021

            wum : (disabled) Wuhan, multi-GNSS, not rapid

            gnss2 : multi-GNSS, but uses IGN instead of CDDIS. does not work

            gnss3 : multi-GNSS, but uses GFZ archive instead of CDDIS. same as gnss-gfz

            ultra : ultra orbits directly from GFZ

            rapid : rapid orbits directly from GFZ

    year : integer
        full year
    month : integer
        calendar month
    day : integer
        day of the month
    doy_end : integer 
        optional, allows multiple day download
    hour : int
        optional hour for ultrarapid orbit , default is zero

    """

#   make sure environment variables exist.  set to current directory if not
    g.check_environ_variables()

    orbit_list = ['igs', 'igr', 'jax', 'grg', 'wum', 'gbm', 'nav', 'gps', 'gps+glo', 
            'gnss', 'gfr', 'esa', 'gnss2', 'gnss3','gnss-gfz','ultra', 'rapid','nav-esa', 'nav-sopac','nav-cddis','wum2']

#   assign to normal variables
    pCtr = orbit

    if len(str(year)) != 4:
        print('Year must have four characters: ', year)
        sys.exit()
    month, day, doy, cyyyy, cyy, cdoy = g.ymd2ch(year,month,day)

    if doy_end == None:
        doy_end = doy 

    if pCtr not in orbit_list:
        print('You picked an orbit type - ', pCtr, ' - that I do not recognize')
        print(orbit_list)
        sys.exit()

    # if generic names used, we direct people to these orbit types
    if pCtr == 'gps':
        pCtr = 'nav'

    # send to CDDIS to get final GFZ orbits
    if pCtr == 'gnss':
        pCtr = 'gbm'

    if pCtr == 'gps+glo':
        pCtr = 'jax'

    # using gfz multi-gnss for rapid
    if pCtr == 'rapid':
        pCtr = 'gfr'

    d1= int(doy); d2 = int(doy_end) + 1
    for d in range(d1, d2):
        s1 = time.time()

        year,month,day= g.ydoy2ymd(year,d)
        print('Looking for ', year, '/', d, ' mm/dd', month, day)
        if (pCtr == 'nav'):
            navname, navdir, foundit = g.getnavfile(year, month, day)
            if foundit:
                print('\n SUCCESS:', navdir+'/'+navname)
        elif (pCtr == 'nav-esa'):
            navname, navdir, foundit = g.getnavfile_archive(year, month, day,'esa')
            if foundit:
                print('\n SUCCESS:', navdir+'/'+navname)
        elif (pCtr == 'nav-cddis'):
            navname, navdir, foundit = g.getnavfile_archive(year, month, day,'cddis')
            if foundit:
                print('\n SUCCESS:', navdir+'/'+navname)
        elif (pCtr == 'nav-sopac'):
            navname, navdir, foundit = g.getnavfile_archive(year, month, day,'sopac')
            if foundit:
                print('\n SUCCESS:', navdir+'/'+navname)
        else:
            if (pCtr == 'igs') or (pCtr == 'igr'):
                filename, fdir, foundit = g.getsp3file_flex(year, month, day, pCtr)
            else:
                if pCtr == 'esa':
                # this is ugly - but hopefully will work for now.
                    filename, fdir, foundit = g.getsp3file_flex(year, month, day, pCtr)
                elif (pCtr == 'wum2'):
                     filename,fdir,foundit = g.get_wuhan_orbits(year,month,day,hour)
                elif (pCtr == 'gfr'):
                # rapid GFZ is available again ...
                # updated yet again, new location/filenames 2024 june 4
                    filename, fdir, foundit = g.rapid_gfz_orbits(year, month, day)
                elif pCtr == 'ultra':
                    if (year + d/365.25) > (2024 + 153/365.25):
                        # use the new filenames and location ... 
                        if (d == 1):
                            # if january 1
                            filename, fdir, foundit = g.new_ultra_gfz_orbits(year-1, 12, 31,hour)
                        else:
                            # give it day of year (minus 1) in the second input ... 
                            filename, fdir, foundit = g.new_ultra_gfz_orbits(year, d-1, 0,hour)
                    else:
                        # should update this
                        filename, fdir, foundit = g.ultra_gfz_orbits(year, month, day, hour)
                elif (pCtr == 'gnss3') or (pCtr == 'gnss-gfz'):
                    # use GFZ ftp site instead of CDDIS
                    if (year >= 2024):
                        filename, fdir, foundit = g.newish_gfz_orbits(year,month,day, 'final')
                    else:
                        filename, fdir, foundit = g.gbm_orbits_direct(year, month, day)
                elif pCtr == 'gnss2':
                # use IGN instead of CDDIS
                    print('To my knowledge, this option no longer works')
                    filename, fdir, foundit = g.avoid_cddis(year, month, day)
                else:
                    filename, fdir, foundit = g.getsp3file_mgex(year, month, day, pCtr)
            if foundit:
                print('SUCCESS:', fdir+'/'+filename)
            else:
                print(filename, ' not found')
        s2 = time.time()
        print('That download took ', np.round(s2-s1,2), ' seconds')


def main():
    args = parse_arguments()
    download_orbits(**args)


if __name__ == "__main__":
    main()



import argparse
import matplotlib.pyplot as plt
import numpy as np
import os
import subprocess
import sys
import time
import wget

import gnssrefl.gps as g


def vegplt(station, tv,winter):
    """
    makes a plot of MP1 multipath metric. Sends to the screen

    Parameters
    ----------
    station : str
        4 ch station name

    tv: np array 
        (year, doy, mp1, mp2) 

    winter : bool
        whether to throw out ~jan-apr and ~oct-dec

    """
    if (winter == 'True'):
        cc = ((tv[:,1] > 105) & (tv[:,1] < 274))
        tv = tv[cc,:]
    plt.figure()
    t = tv[:,0]+tv[:,1]/365.25
    ii = (tv[:,2] > 0) # MP12
    jj = (tv[:,3] > 0) # MP1
    plt.plot(t[ii],-tv[ii,2],'b.',label='MP12')
    plt.plot(t[jj],-tv[jj,3],'r.',label='MP1')
    plt.ylabel('-L1 Multipath rms(m)')
    plt.title(station + ' preliminary Vegetation statistic')
    plt.grid()
    plt.show()


def sfilename(station, year, doy):
    """
    Finds mp1 filename on your system

    Parameters
    ----------
    station : string
        4 character station name 
    year : integer

    doy : integer
        day of year

    Returns
    -------
    xfile : string
        the full SNR filename on your local system
    """
    cdoy = '{:03d}'.format(doy)
    cyyyy = str(year)
    cyy = cyyyy[2:4]
    ddir = os.environ['REFL_CODE'] + '/' + cyyyy + '/mp/' + station + '/'

    xfile = ddir+ station + cdoy + '0.' + cyy + 'S'

    return xfile

def ReadRecAnt(teqclog):
    """
    prints out Receiver and Antenna name from a teqc log

    Parameters
    ----------
    teqclog : str
        the name of a teqc log 

    """
    mp1 = 0
    mp2 = 0
    foundRec = False
    rinexfile = teqclog
    if os.path.isfile(teqclog):
        f=open(rinexfile,'r')
        lines = f.read().splitlines(True)
        lines.append('')

        for i,line in enumerate(lines):
            if "Receiver type" in line:
                print(line[0:60])
            if "Antenna type" in line:
                print(line[0:60])
            # this assumes Antenna is after Receiver
                break
        f.close()
    else:
        print('File does not exist:', teqclog)

def readoutmp(teqcfile,rcvtype):
    """
    teqcfile input is the full name of a teqc log 
    rcvtype is a string that includes the name of the receiver you are searching
    for. it does not have to be exact (so NETR would work for NETRS)
    returns MP1, MP2 (both in meters), and a boolean as to whether the values were found
    if rcvtype is set to NONE, it will return data without restriction 
    """
    # change variable be ause i am reusing old code
    rinexfile = teqcfile

    f=open(rinexfile,'r')
    lines = f.read().splitlines(True)
    lines.append('')

    # defaults  so there is something to return
    mp12 = '0'
    mp1 = '0'
    rcvtypeinfile = 'xxxx'
    # set this to always be true if you don't want to restrict to one receiver
    if rcvtype == 'NONE':
        foundRec = True
    else:
        foundRec = False

    for i,line in enumerate(lines):
        if "Receiver type" in line:
            # remove white space
            rcvtypeinfile=line[26:40].replace(" ","")
        # new style teqc
        if "Moving average MP12" in line:
            mp12 = line[26:34].strip()
        # old style teqc
        if "Moving average MP1 " in line:
            mp1 = line[26:34].strip()
        if  rcvtype in line:
            foundRec = True
    f.close()
    return mp12, mp1, foundRec, rcvtypeinfile


def run_teqc(teqc,navfile,rinexfile,foutname,mpdir):
    """
    run teqcs and stores the output 

    Parameters
    ----------
    teqc : str
        location of the teqc executable
    navfile : str
        name of the RINEX nav file
    rinexfile : string
        name of the RINEX observation file
    foutname : str
        name of the output file
    mpdir : str
        location of the multipath directory on your system

    """
    line = [teqc, '-nav', navfile, '+qc', rinexfile]
    subprocess.call(line)
    details = rinexfile[0:11] + 'S'
    print('SUCCESS: ',mpdir + '/' + details)
    subprocess.call(['mv','-f',details,mpdir] )
    # clean up - remove rinex file
    subprocess.call(['rm','-f',rinexfile] )

def check_directories(station,year):
    """
    checks that directories exist for teqc logs

    Parameters
    ----------
    station : str
        4 character station name

    year : int
        full year

    """
    navfiledir = os.environ['ORBITS']  + '/' + str(year)
    if not os.path.isdir(navfiledir):
        subprocess.call(['mkdir',navfiledir] )

    navfiledir = os.environ['ORBITS']  + '/' + str(year) + '/nav'
    if not os.path.isdir(navfiledir):
        subprocess.call(['mkdir',navfiledir] )

    foutdir =  os.environ['REFL_CODE'] + '/' + str(year) 
    if not os.path.isdir(foutdir):
        subprocess.call(['mkdir',foutdir] )

    foutdir =  os.environ['REFL_CODE'] + '/' + str(year) + '/mp'
    if not os.path.isdir(foutdir):
        subprocess.call(['mkdir',foutdir] )

    foutdir =  os.environ['REFL_CODE'] + '/' + str(year) + '/mp/' + station
    if not os.path.isdir(foutdir):
        subprocess.call(['mkdir',foutdir] )

    return navfiledir, foutdir


def get_files(station,year,doy,look):
    """
    Parameters
    ----------
    station : str
        4 character station name
    year : int
        full year
    doy : int
        day of year
    look : bool
        whether you should try to get the file from unavco 
        if it does not exist locally

    Returns
    -------
    navfile : str
        navigation/orbit file
    rinexfile : str
        name of the obs file
    foutname : str
        full name of the teqc log output 
    mpdir :  str
        directory for MP results
    goahead : boolean
        whether you should go ahead and run teqc

    """
    goahead = False # means you found everything and can run teqc

    # check that directories exist. Make them if they do not
    navfiledir, foutdir = check_directories(station,year)
    mpdir = foutdir

    cdoy = '{:03d}'.format(doy) ; 
    cyy = '{:02d}'.format(year-2000)
    cyyyy = '{:04d}'.format(year-2000)

    navfile = navfiledir + '/' + 'auto' + cdoy + '0.' + cyy + 'n'
    nd = navfile + '.xz'

    if not os.path.isfile(navfile):
        # it is either zipped
        if os.path.isfile(nd):
            subprocess.call(['unxz',nd] )
        # or you should go get it
        else:
            n1,n2,foundit = g.getnavfile(year, doy, 0)

    rinexfile = station + cdoy + '0.' + cyy + 'o'
    rexist = os.path.isfile(rinexfile)
    nexist = os.path.isfile(navfile)

    if not rexist:
        print('No RINEX file',rinexfile)
        if look == None:
            goahead = False
        else:
            g.rinex_unavco(station, year, doy,0)
            if os.path.isfile(rinexfile):
                goahead = True
                print('retrieved from UNAVCO: ',rinexfile)
    else:
        goahead = True

    if not nexist:
        goahead = False
        print('No nav file',navfile)

    foutname = foutdir + '/' + station + cdoy + '0.' + cyy + '.mp'
    print(navfile, rinexfile,foutname,mpdir)
    return navfile, rinexfile,foutname,mpdir, goahead

def main():
    """
    computes MP1 MP2 stats using teqc or reads existing log

    """

    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="station name", type=str)
    parser.add_argument("year", help="year", type=int)
    parser.add_argument("doy", help="day of year", type=int)
    parser.add_argument("-doy_end", default = None, help="day of year - to analyze multiple days", type=int)
    parser.add_argument("-year_end", default = None,  type=int)
    parser.add_argument("-rcvant", default = None, help="True to output receiver/antenna type", type=str)
    parser.add_argument("-look", default = None, help="True to try and download from unavco", type=str)

    args = parser.parse_args()

#   make sure environment variables exist.  set to current directory if not
    g.check_environ_variables()

#   assign to normal variables
    station = args.station
    if len(station) != 4:
        print('illegal station - must be 4 char')
        sys.exit()

    year = args.year
    doy = args.doy
    if args.doy_end == None:
        doy_end = doy + 1
    else:
        doy_end = args.doy_end + 1

    # check the receiver and antenna in the teqc file
    if args.rcvant != None: 
        s = sfilename(station, year, doy)
        ReadRecAnt(s)
        sys.exit()


    # get the name/location of the teqc executable
    teqc = g.teqc_version()

    if not os.path.isfile(teqc):
        print('teqc has to be installed in EXE. Exiting. ')
        sys.exit()
    else:
           # run multiple days for a station within a given year
        for d in range(doy,doy_end):
            navfile, rinexfile,foutname,mpdir,goahead = get_files(station,year,d,args.look)
            if goahead:
                run_teqc(teqc,navfile,rinexfile,foutname,mpdir)

if __name__ == "__main__":
    main()





# -*- coding: utf-8 -*-
"""
"""
import argparse
import os
import subprocess
import sys

import gnssrefl.gps as g
import gnssrefl.rinex2snr as r
import gnssrefl.gnsssnrbigger as gnsssnrbigger


def main():
    """

    Creates SNR file from RINEX 3 file that is stored locally in makan folders
    It may allow crx - I am not sure.
    Only allows GFZ ultra orbit files

    Parameters
    ----------
    rinex3 : str
        name of filename

    """

    parser = argparse.ArgumentParser()
    parser.add_argument("rinex3", help="rinex3 filename", type=str)
    parser.add_argument("-dec", help="decimation", default=None,type=str)


    args = parser.parse_args()
    rinex3 = args.rinex3

    station = rinex3[0:4].lower()
    STATION = rinex3[0:4]

    year = rinex3[12:16]
    iyear = int(year)
    cdoy = rinex3[16:19] 
    idoy = int(cdoy)
    iyear,month,day=g.ydoy2ymd(iyear, idoy)

    maindir = os.environ['REFL_CODE']
    # where rinex data are
    xdir = os.environ['REFL_CODE'] + '/rinex/' + STATION + '/' + year + '/'
    # where snr data will go
    snrdir =  os.environ['REFL_CODE'] + '/' + year + '/snr/' + STATION + '/'
    print(xdir)


    # default keeps everything
    if args.dec is None:
        dec_rate = 1
    else:
        dec_rate = int(args.dec)

    # all lowercase in my world
    rinex2 = station + cdoy + '0.' + year[2:4] + 'o'

    # we only support GFZ ultra for now
    orbtype = 'ultra'
    hour = 0 # for now only download hour 0 for ultra products
    filename, fdir, foundit = g.ultra_gfz_orbits(iyear, month, day, hour)
    orbfile = fdir + '/' + filename
    if not os.path.exists(orbfile):
        print('Required orbitfile does not exist. Exiting.')
        sys.exit()


    gexe = g.gfz_version()

    if not os.path.exists(gexe):
        print('Required gfzrnx executable does not exist. Exiting.')
        sys.exit()


    # first step will be to translate to rinex2

    full_rinex3 = xdir + rinex3
    if os.path.isfile(full_rinex3):
        print('found version 3 rinex in the makan directories')
        g.new_rinex3_rinex2(full_rinex3,rinex2)
    else:
        print('ERROR: your input file does not exist:', rinex3)
        sys.exit()

    g.make_snrdir(year,STATION) # make sure output directory exists
    snrname = snrdir + rinex3[0:-3] + 'snr66'

    in1 = g.binary(rinex2)
    in2 = g.binary(snrname) 
    in3 = g.binary(orbfile)

    if (len(snrname) > 132) or (len(orbfile) > 132):
        print('The orbit or SNR file name is too long.')
        print('Make your environment variable names shorter.')
        sys.exit()
    option = 66
    in4 = g.binary(str(option))
    if (dec_rate > 0):
        decr = str(dec_rate)
    else:
        decr = '0'
    in5 = g.binary(decr) # decimation can be used in hybrid option
    message = 'None '
    errorlog = maindir + '/logs/' + station + '_hybrid_error.txt'
    in6 = g.binary(errorlog)
    gnsssnrbigger.foo(in1,in2,in3,in4,in5,in6)
    # clean up - remove the rinex2 file
    print('Output file written to: ', snrname)
    subprocess.call(['rm','-f',rinex2])

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
download a year of teqc logs from unavco
can do multiple years as well
2022 september 15, updated to https access
"""
import argparse
import os
import subprocess
import sys

import wget

import gnssrefl.gps as g
import gnssrefl.computemp1mp2 as veg


def mpfile_unavco(station, year, doy):
    """
    picks up teqc log from unavco if it exists
    stores it in $REFL_CODE / year / mp / station directory
    does not check that directory exists.  Assumes you previously
    ran check_directories from the veg library

    Parameters
    ----------
    station : string
        four character station name

    year : integer

    doy : integer
        day of year

    """
    cdoy = '{:03d}'.format(doy)
    cyyyy = str(year)
    cyy = cyyyy[2:4]

    # info for unavco 
    # - changed to https 2022 september 15
    fdir = 'https://data.unavco.org/archive/gnss/rinex/qc/'
    #       https://data.unavco.org/archive/gnss/rinex/qc/2010/001/
    # old way
    #fdir = 'ftp://data-out.unavco.org/pub/rinex/qc/'
    fdir = fdir + cyyyy + '/' + cdoy + '/'
    fname = station + cdoy + '0.' + cyy + 'S'
    url = fdir + fname

    # local directory info
    ddir = os.environ['REFL_CODE'] + '/' + cyyyy + '/mp/' + station + '/'

    if not os.path.isdir(ddir):
        print('Required output directory does not exist. ', ddir)
        print('This should have been created by download_teqc')
        sys.exit()

    print('Looking for: ', url)
    print('Will store in: ', ddir)

    if os.path.isfile(ddir + fname):
        print('teqc log already exists', ddir+fname)
    else:
        try:
            wget.download(url,out=fname)
        except:
            print('download failed for ', fname)
        if os.path.isfile(fname):
           subprocess.call(['mv', fname, ddir])
           print('\n SUCCESS', ddir + '/' + fname)


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="station name", type=str)
    parser.add_argument("year", help="year", type=int)
    parser.add_argument("-year_end", default=None, help="end year", type=int)

    args = parser.parse_args().__dict__

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def download_teqc(station: str, year: int, year_end: int = None):
    """
    Download teqc logs from UNAVCO for one (or more) year.
        
    Parameters
    ----------
    station : string
        4 character ID of the station

    year : integer
        Year

    year_end : int, optional
        end year. 

    """

#   make sure environment variables exist.  set to current directory if not
    g.check_environ_variables()

    if len(station) != 4:
        print('illegal station - must be 4 char')
        sys.exit()

    y1 = year; 
    if year_end is None:
        y2 = year + 1
    else:
        y2 = year_end + 1
    for y in range(y1, y2):
        veg.check_directories(station, y)
        # life is short - assume it is always a leap year
        for d in range(1, 367):
            mpfile_unavco(station, y, d)


def main():
    args = parse_arguments()
    download_teqc(**args)


if __name__ == "__main__":
    main()



import argparse
import json
import os
import subprocess
import sys

import gnssrefl.gps as g
import gnssrefl.gnssir_v2 as guts2

from gnssrefl.utils import str2bool


def parse_arguments():
    # user inputs the observation file information
    parser = argparse.ArgumentParser()
    # required arguments
    parser.add_argument("station", help="station (lowercase)", type=str)
    parser.add_argument("-lat", help="Latitude (degrees)", type=float)
    parser.add_argument("-lon", help="Longitude (degrees)", type=float)
    parser.add_argument("-height", help="Ellipsoidal height (meters)", type=float)
    parser.add_argument("-e1", default=None, type=float, help="Lower limit elevation angle (deg),default 5")
    parser.add_argument("-e2", default=None, type=float, help="Upper limit elevation angle (deg), default 25")
    parser.add_argument("-h1", default=None, type=float, help="Lower limit reflector height (m), default 0.5")
    parser.add_argument("-h2", default=None, type=float, help="Upper limit reflector height (m), default 8")
    parser.add_argument("-nr1",default=None, type=float, help="Lower limit RH used for noise region in QC(m)")
    parser.add_argument("-nr2",default=None, type=float, help="Upper limit RH used for noise region in QC(m)")
    parser.add_argument("-peak2noise", default=None, type=float, help="peak to noise ratio used for QC")
    parser.add_argument("-ampl", default=None, type=float, help="Required spectral peak amplitude for QC")
    parser.add_argument("-allfreq", default=None, type=str, help="Set to T to include all GNSS signals")
    parser.add_argument("-l1", default=None, type=str, help="Set to T to use only GPS L1")
    parser.add_argument("-l2c", default=None, type=str, help="Set to T to only use GPS L2C")
    parser.add_argument("-xyz", default=None, type=str, help="Set to True if using Cartesian coordinates instead of LLH")
    parser.add_argument("-refraction", default=None, type=str, help="Set to False to turn off refraction correction")
    parser.add_argument("-extension", default=None,type=str, help="Provide extension name so you can try different strategies")
    parser.add_argument("-ediff", default=None, type=float, help="Allowed min/max elevation diff from obs min/max elev angle (degrees) default is 2")
    parser.add_argument("-delTmax", default=None, type=float, help="max arc length (min) default is 75. Shorten for tides.")
    parser.add_argument("-frlist", nargs="*",type=int,  help="User defined frequencies using our nomenclature, e.g. 1 101 for GPS and Glonass L1")
    parser.add_argument("-azlist2", nargs="*",type=float,  default=None,help="list of azimuth regions, default is 0-360") 
    parser.add_argument("-ellist", nargs="*",type=float,  default=None,help="List of elevation angles, advanced users only!") 
    parser.add_argument("-refr_model", default="1", type=str, help="refraction model. default is 1, zero turns it off)")
    parser.add_argument("-apriori_rh", default=None, type=float, help="apriori reflector height (m) used by NITE model")
    parser.add_argument("-Hortho", default=None, type=float, help="station orthometric height (m)")
    parser.add_argument("-pele", nargs="*", type=float, help="min and max elevation angle in direct signal removal, default is 5-30")
    parser.add_argument("-daily_avg_medfilter", default=None, type=float, help="daily_avg, median filter, meters")
    parser.add_argument("-daily_avg_reqtracks", default=None, type=int, help="daily_avg, ReqTracks parameter ")
    parser.add_argument("-subdaily_alt_sigma", default=None, type=str, help="subdaily, Nievinski sigma")
    parser.add_argument("-subdaily_sigma", default=None, type=str, help="subdaily, sigma for part 1")
    parser.add_argument("-subdaily_ampl", default=None, type=float, help="subdaily, LSP amplitude override")
    parser.add_argument("-subdaily_delta_out", default=None, type=int, help="subdaily, spline output interval (sec)")
    parser.add_argument("-subdaily_knots", default=None, type=float, help="subdaily, knots, how many per day")
    parser.add_argument("-subdaily_subdir", default=None, type=str, help="subdaily, output directory")
    parser.add_argument("-subdaily_spline_outlier1", default=None, type=float, help="subdaily, outlier value (m), part1")
    parser.add_argument("-subdaily_spline_outlier2", default=None, type=float, help="subdaily, outlier value (m), part2")

    args = parser.parse_args().__dict__

    g.print_version_to_screen()

    # convert all expected boolean inputs from strings to booleans
    boolean_args = ['allfreq', 'l1', 'l2c', 'xyz', 'refraction','subdaily_alt_sigma']
    args = str2bool(args, boolean_args)

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def make_gnssir_input(station: str, lat: float=0, lon: float=0, height: float=0, e1: float = 5.0, e2: float = 25.0,
       h1: float = 0.5, h2: float = 8.0, nr1: float = None, nr2: float = None, peak2noise: float = 2.8, 
       ampl: float = 5.0, allfreq: bool = False, l1: bool = False, l2c: bool = False, 
       xyz: bool = False, refraction: bool = True, extension: str = '', ediff: float=2.0, 
       delTmax: float=75.0, frlist: list=[], azlist2: list=[0,360], ellist : list=[], refr_model : str="1", 
                      apriori_rh: float=None, Hortho : float = None, pele: list=[5,30], daily_avg_reqtracks: int=None, 
                      daily_avg_medfilter: float =None, subdaily_alt_sigma : bool=None, 
                      subdaily_ampl : float=None, subdaily_delta_out : float=None, 
                      subdaily_knots : int=None, subdaily_sigma: float=None, subdaily_subdir: str=None, 
                      subdaily_spline_outlier1: float=None, subdaily_spline_outlier2: float=None):

    """
    This new script sets the Lomb Scargle analysis strategy you will use in gnssir. It saves your inputs 
    to a json file which by default is saved in REFL_CODE/<station>.json. This code replaces make_json_input.

    This version no longer requires you to have azimuth regions of 90-100 degrees. You can set a single set of 
    azimuths in the command line variable azlist2, i.e. -azlist2 0 270 would accommodate all rising and setting arcs 
    between 0 and 270 degrees. If you have multiple distinct regions, that is also acceptable, i.e. -azlist2 0 150 180 360  
    would use all azimuths between 0 and 360 except for 150 to 180

    Your first azimuth constraint can be negative, i.e. -azlist2 -90 90, is allowed.

    Note: you can keep using your old json files - you just need to add this new -azlist2 setting manually.

    Latitude, longitude, and height are assumed to be stored in the UNR database.  If they are not, you should
    set them manually.

    Originally we had refraction as a boolean, i.e. on or off. This was stored in the gnssir 
    analysis description json. The code however, uses a 1 for a simple non-time-varying 
    Bennett correction and 0 for no correction.

    From version 1.8.4 we begin to implement more refraction models.  Model 1 (Bennett) will continue to be 
    the default.  The model number is written (as an integer) to the LSP results file so that people can keep track easily of whether
    they are inadvertently mixing files with different strategies. And that is why it is an integer, because
    all results in the LSP results files are numbers.  Going forward, we are adding a time-varying capability.

        Model 1: Bennett, non-time-varying

        Model 2: Bennett, time-varying

        Model 3: Ulich, non-time-varying

        Model 4: Ulich, time-varying

        Model 5: NITE, Feng et al. 2023 DOI: 10.1109/TGRS.2023.3332422, time-varying

        Model 6: MPF, Williams and Nievinski, 2017,  DOI: 10.1002/2016JB013612, time-varying


    We allow users to input the model names NITE and MPF (nite and mpf also allowed).  The other models do not allow that.
    If you want model 2, you have to ask for model 2. We thank Peng Feng for providing python code to be used for some of these models.

    If you want to test the effect of different refraction models, you are encouraged to create
    two json files using the extension option. You can then run gnssir using those two extensions.
    In general I think the refraction default is fine for soil moisture and snow accumulation.
    If you are going to look at tall sites, you most definitely need the refraction correction. If you
    plan to look at very tall sites, you should pick the best one.

    Examples
    --------
    gnssir_input p041 
        uses only GPS frequencies and all azimuths and the coordinates in the UNR database

    gnssir_input p041   -azlist2 0 180 -fr 1 101
        uses UNR coordinates, GPS L1 and Glonass L1 frequencies, and azimuths between 0 and 180.

    gnssir_input p041 -lat 39.9494 -lon -105.19426 -height 1728.85  -l2c T -e1 5 -e2 15
        uses only L2C GPS data between elevation angles of 5 and 15 degrees.
        user input lat/long/height

    gnssir_input p041  -h1 0.5 -h2 10 -e1 5 -e2 25
        uses UNR database, only GPS data between elevation angles of 5-25 degrees and reflector heights of 0.5-10 meters

    gnssir_input p041 -ediff 1
        uses UNR database, only GPS data, default station coordinates, enforces elevation angles to be 
        within 1 degrees of default elevation angle limits (5-25)

    gnssir_input sc02 -ellist 5 10 7 12
        let's say you want to compute smaller arcs than just a single set of elevation angles.
        you can use this to set this up, so instead of 5 and 12, you could set it up to 
        do two arcs, one for 5-10 degrees and the other for 7-12.
        WARNING: you need to pay attention to QC metrics (amplitude and peak2noise).  You likely need to lower them since 
        your periodogram for fewer data will be less robust than with the longer elevation angle region.

    Parameters
    ----------
    station : str
        4 character station ID.

    lat : float, optional
        latitude in degrees.

    lon : float, optional
        longitude in degrees.

    height : float, optional
        ellipsoidal height in meters.

    e1 : float, optional
        elevation angle lower limit in degrees. default is 5.

    e2 : float, optional
        elevation angle upper limit in degrees. default is 25.

    h1 : float, optional
        reflector height lower limit in meters. default is 0.5.

    h2 : float, optional
        reflector height upper limit in meters. default is 8.

    nr1 : float, optional
        noise region lower limit for QC in meters. default is None.

    nr2 : float, optional
        noise region upper limit for QC in meters. default is None.

    peak2noise : float, optional
        peak to noise ratio used for QC.
        default is 2.7 (just a starting point for water - should be 3 or 3.5 for snow or soil...)

    ampl : float, optional
        spectral peak amplitude for QC. default is 6.0
        this is receiver and elevation angle region dependent - so you need to change it based on your site 

    allfreq : bool, optional
        True requests all GNSS frequencies.
        default is False (defaults to use GPS frequencies).

    l1 : bool, optional
        set to True to use only GPS L1 frequency. default is False.

    l2c : bool, optional
        set to use only GPS L2C frequency. default is False.

    xyz : bool, optional
        set to True if using Cartesian coordinates instead of Lat/Long/Ht.
        default is False.

    refraction : bool, optional
        set to False to turn off refraction correction.
        default is True.

    extension : str, optional
        provide extension name so you can try different strategies. 
        Results will then go into $REFL_CODE/YYYY/results/ssss/extension
        Default is '' 

    ediff : float, optional
        quality control parameter (Degrees)
        Allowed min/max elevation angle diff from requested min/max elev angle
        default is 2

    delTmax : float, optional
        maximum allowed arc length (minutes)
        default is 75, which can be a bit long for tides

    frlist : list of integers
        avoids all the booleans - if you know the frequencies, enter them.
        e.g. 1 2 or 1 20 5 or 1 20 101 102

    azlist2 : list of floats
        Default is 0 to 360. list of azimuth limits as subquadrants are no longer required.

    ellist: list of floats
        min and max elevation angles to be used with the azimuth regions you listed, i.e.
        [5 10 6 11 7 12 8 13] would allow overlapping regions - all five degrees long 
        Default is empty list. 

    refr_model : str
        refraction model. While defined as a string (so that people can specify names of models)
        we convert this to an integer for book-keeping. 1 is the default refraction model 
        (it corrects elevation angles using standard bending models).  0 is no refraction correction.  
        The other models are defined in the summary section of this code.

    apriori_rh : float
        apriori reflector height (meters). only used in NITE model 

    Hortho : float
        station orthometric height, in meters. Currently only used in subdaily.  If not provided on the command line, 
        it will use ellipsoidal height and EGM96 to compute.

    pele : float
        min and max elevation angles in direct signal removal, i.e. 3 40. Default is 5 30. 

    daily_avg_reqtracks : int, optional
        number of tracks required for daily_avg code

    daily_avg_medfilter : float, optional
        median filter value required for daily_avg code (meters)

    subdaily_alt_sigma : bool, optional
        use Nievinski sigma definition

    subdaily_ampl : float, optional
        override the required LSP amplitude  

    subdaily_delta_out : int, optional
        spacing for final subdaily spline output

    subdaily_knots : int, optional
        number of knots per day for subdaily spline fits

    subdaily_sigma : float, optional
        how many standard deviations for outliers in subdaily code setting

    subdaily_subdir : str, optional
        non-standard location for subdaily outputs

    subdaily_spline_outlier1 : float, optional
        alternate setting for outlier detection in part1

    subdaily_spline_outlier2 : float, optional
        alternate setting for outlier detection in part2

    """

    # make sure environment variables exist
    g.check_environ_variables()

    ns = len(station)
    if ns != 4:
        print('station name must be four characters long. Exiting.')
        sys.exit()


# location of the site - does not have to be very good.  within 100 meters is fine
    query_unr = False
# if you input lat and long as zero ...
    if lat + lon == 0:
        print('Assume you want to use our database for a priori receiver coordinates.')
        query_unr = True

    if xyz:
        xyz = [lat, lon, height]
        print(xyz)
        lat, lon, height = g.xyz2llhd(xyz)
        print(lat,lon,height)

    if query_unr:
        # try to find the coordinates  at UNR
        lat, lon, height = g.queryUNR_modern(station)
        if lat == 0:
            print('Tried to find coordinates in our station database. None found so exiting')
            sys.exit()

    # calculate Hortho using EGM96 if none provided
    if Hortho is None:
        geoidC = g.geoidCorrection(lat,lon)
        Hortho = height - geoidC

    if apriori_rh is None:
        apriori_rh = 5 # completely made up, meters

# start the lsp dictionary
    reqA = ampl

    lsp = {}
    lsp['station'] = station.lower()
    lsp['lat'] = lat
    lsp['lon'] = lon
    lsp['ht'] = height
    lsp['Hortho'] = round(Hortho,4) # no point having it be so many decimal points
    lsp['apriori_rh'] = apriori_rh
    

    if h1 > h2:
        print(f'h1 cannot be greater than h2. You have set h1 to {h1} and h2 to {h2}. Exiting.')
        sys.exit()

    if ( (h2-h1)  < 5):
        print(f'h2-h1 must be at least 5 meters apart. You have set h1 to {h1} and h2 to {h2}. Exiting.')
        sys.exit()

    lsp['minH'] = h1 ; lsp['maxH'] = h2 ; lsp['e1'] = e1 ; lsp['e2'] = e2

# the default noise region will the same as the RH exclusion area for now
    if nr1 is None:
        nr1 = h1
    if nr2 is None:
        nr2 = h2

    lsp['NReg'] = [nr1, nr2]
    lsp['PkNoise'] = peak2noise

    # where the instructions will be written
    xdir = os.environ['REFL_CODE']
    outputdir = xdir + '/input'
    if not os.path.isdir(outputdir):
        subprocess.call(['mkdir', outputdir])

    if len(extension) == 0:
        outputfile = outputdir + '/' + station + '.json'
    else:
        outputfile = outputdir + '/' + station + '.' + extension + '.json'

    lsp['polyV'] = 4 # polynomial order for DC removal
    # change this so the min elevation angle for polynomial removal is the same as the 
    # requested analysis region. previously it was hardwired to 5-30

    lsp['pele'] = pele # elevation angles used for DC removal

    default_pele1 = 5; default_pele2 = 30
    if (pele[0] == default_pele1) & (pele[1] == default_pele2):
        print('Using default DC removal elevation angle limits,', default_pele1, default_pele2)
        print('Checking that they are sensible')
        usethis1 = default_pele1; usethis2 = default_pele2 
        if (lsp['e1']) < default_pele1:
            usethis1 = lsp['e1']
        if (lsp['e2']) > default_pele2:
            usethis2 = lsp['e2']
        lsp['pele'] = [usethis1, usethis2] # modified elevation angles 
    else:
        print('You manually set the DC removal elevation angle limits ', pele)
        print('We will respect your wishes. For future reference, ')
        print('they do not need to be the same as the requested elevation ')
        print('angle range - but they cannot be outside of them. For example,')
        print('pele min cannot be greater than e1 and pele max cannot be less than e2.')

    #if ((lsp['e1'] < 5 or lsp['e2'] > 30) and (lsp['pele'][0] >= 5 and lsp['pele'][1] <= 30)):
    ## Check if the elevation angle limits for DC removal are outside the default range
    #    print('Change the pele (elevation angle limits) for DC removal')
    #    sys.exit()
    #else:
    #    lsp['pele'] = pele # elevation angles used for DC removal

    lsp['ediff'] = ediff # degrees
    lsp['desiredP'] = 0.005 # precision of RH in meters
    # azimuth regions in degrees (in pairs)
    # you can of course have more subdivisions here
    #if (az_list[0]) == 0 & (az_list[-1] == 360):

    N2 = len(azlist2)
    if (N2 % 2) == 0:
        lsp['azval2'] = azlist2
    else:
        print('Illegal azimuth inputs. Must be an even number of azimuth pairs.')
        sys.exit()

    # default frequencies to use - and their required amplitudes. The amplitudes are not set in stone
    # this is the case for only GPS, but the good L2 
    lsp['freqs'] = [1, 20, 5]
    if allfreq is True:
        # 307 was making it crash.  did not check as to why
        # includes glonass, galileo, and beidou
        lsp['freqs'] = [1, 20, 5, 101, 102, 201, 205, 206, 207, 208, 302, 306]

    if l1 is True:
        lsp['freqs'] = [1]

    if l2c is True:
        lsp['freqs'] = [20]

    if len(frlist) == 0:
        print('Using standard frequency choices.')
    else:
        print('Implementing user-provided frequency list.')
        lsp['freqs'] = frlist

    # create a list with all values equal to reqA
    # but the length of the list depends on the length of the list of frequencies
    lsp['reqAmp'] = [reqA]*len(lsp['freqs'])

    # this is true or false.  I think
    lsp['refraction'] = refraction

    # write new RH results  each time you run the code
    lsp['overwriteResults'] = True

    # if snr file does not exist, try to make one
    # i don't think this option exists anymore
    lsp['seekRinex'] = False

    # compress snr files after analysis - saves disk space
    lsp['wantCompression'] = False

    # periodogram plots come to the screen
    lsp['plt_screen'] = False

    # command line req to only do a single satellite - default is do all satellites
    lsp['onesat'] = None

    # default will now be False ....
    # send some information on periodogram RH retrievals to the screen
    lsp['screenstats'] = False

    # save the output plots
    lsp['pltname'] = station + '_lsp.png'

    # how long can the arc be, in minutes
    lsp['delTmax'] = delTmax  
 
    # gzip SNR files after running the code
    # this really should be set to True.  the code is obviously ignoring it
    lsp['gzip'] = False   

    lsp['ellist'] = ellist

    if refr_model[0] == '-':
         print('You selected a negative refraction model. Exiting')
         sys.exit()

    if refr_model.isnumeric():
        # number has been input
        refr_model = int(refr_model)
        if (refr_model > 6) :
            print('We only have six refraction models and you selected ', refr_model, ' Exiting')
            sys.exit()
    else:
        if refr_model.upper() == 'NITE':
            refr_model = 5
        elif refr_model.upper() == 'MPF':
            refr_model = 6
        else:
            print('Your refraction model ', refr_model, ' is not recognized by the code. Exiting')

    lsp['refr_model'] = refr_model
    print('refraction model ', refr_model)

    # added for people that want to save their daily average and subdaily strategies.
    # if not set, then they are saved as None.
    lsp['daily_avg_reqtracks'] = daily_avg_reqtracks
    lsp['daily_avg_medfilter'] = daily_avg_medfilter

    lsp['subdaily_alt_sigma'] = subdaily_alt_sigma
    lsp['subdaily_ampl'] = subdaily_ampl
    lsp['subdaily_delta_out'] = subdaily_delta_out
    lsp['subdaily_knots'] = subdaily_knots
    lsp['subdaily_sigma'] = subdaily_sigma
    lsp['subdaily_spline_outlier1'] = subdaily_spline_outlier1
    lsp['subdaily_spline_outlier2'] = subdaily_spline_outlier2
    lsp['subdaily_subdir'] = subdaily_subdir

    print('writing out to:', outputfile)
    print(lsp)
    with open(outputfile, 'w+') as outfile:
        json.dump(lsp, outfile, indent=4)


def main():
    args = parse_arguments()
    make_gnssir_input(**args)


if __name__ == "__main__":
    main()



import argparse
import matplotlib.pyplot as matplt
import numpy as np
import datetime 
import os
import sys

import gnssrefl.gps as g
import gnssrefl.snow_functions as sf
import gnssrefl.daily_avg_cl as da

from gnssrefl.utils import str2bool

def parse_arguments():
    # must input start and end year
    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="station name (4 ch only)", type=str)
    parser.add_argument("year", help="Northern Hemisphere water year", type=int)
    parser.add_argument("-minS", help="y-axis minimum snow depth (m)", type=float,default=None)
    parser.add_argument("-maxS", help="y-axis maximum snow depth (m)", type=float,default=None)
    parser.add_argument("-longer", help="plot longer series", type=str, default=None)
    parser.add_argument("-bare_date1", help="bare soil start yyyy-mm-dd", type=str, default=None)
    parser.add_argument("-bare_date2", help="bare soil end yyyy-mm-dd", type=str, default=None)
    parser.add_argument("-plt_enddate", help="end date for the plot/snow solns, yyyy-mm-dd", type=str, default=None)
    parser.add_argument("-plt", help="whether you want the plot to come to the screen", type=str, default=None)
    parser.add_argument("-simple", help="use simple algorithm (default is false)", type=str, default=None)
    parser.add_argument("-medfilter", help="median filter for daily average(m)", type=float, default=None)
    parser.add_argument("-ReqTracks", help="how many arcs needed for daily average)", type=int, default=None)
    parser.add_argument("-fr", help="if you want to restrict to a single frequency", type=int, default=None)
    parser.add_argument("-barereq_days", help="how many bare soil values req (default is 15)", type=int, default=None)
    parser.add_argument("-hires_figs", help="if you want eps instead of png plots", type=str, default=None)

    args = parser.parse_args().__dict__

    # convert all expected boolean inputs from strings to booleans
    boolean_args = ['longer','plt','simple','hires_figs']
    args = str2bool(args, boolean_args)

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def snowdepth(station: str, year: int, minS: float=None, maxS: float=None,
        longer:bool=False, plt:bool=True, bare_date1:str=None, bare_date2:str=None, plt_enddate:str=None, simple:bool=False, 
              medfilter:float = None, ReqTracks: int = None, barereq_days: int = 15, 
              fr: int = None, hires_figs : bool = False):
    """
    Calculates snow depth for a given station and water year.
    Before you run this code you must have run gnssir for each day of interest.  

    You can then run daily_avg to concatenate the results or you can input appropriate 
    values to optional inputs medfilter and ReqTracks.  

    Currently set for northern hemisphere constraints. This could easily be fixed for 
    the southern hemisphere by reading the json input file.
    Default values use the median of September results to set "bare soil value"
    These can be overriden with bare_date1 and bare_date2 (as one would do in Alaska)

    Output is currently written to a plain text file and a plot is written to a png file.
    Both are located in the $REFL_CODE/Files/station directory

    If simple is set to true, the algorithms computes bare soil (and thus snow depth), using
    all values together.  The default defines bare soil values every 10 degrees in azimuth.  

    2024 Feb 6 : stopped the code from setting snowdepth values < 5 cm to zero. This means that
    "negative" snowdepth will be in the files, but it should not be interpreted to be a new
    form of snow.

    Examples
    --------
    snowdepth p101 2022
        would use results from a previous run of daily_avg

    snowdepth p101 2022 -medfilter 0.25 -ReqTracks 50
        would run daily_avg for you using 50 tracks/0.25 meter median filter

    Parameters
    ----------
    station : str
        4 character station name
    year : int
        water year (i.e. jan-june of that year and oct-dec of the previous year)
    minS : float, optional
        minimum snow depth for y-axis limit (m), optional
    maxS : float, optional
        maximum snow depth for y-axis limit (m), optional
    longer : bool, optional
        whether you want to plot longer time series (useful for Alaskan sites)
    plt : bool, optional
        whether you want the plot to come to the screen
    bare_date1: str, optional
        an override for start bare soil definition 
    bare_date2: str, optional
        an override for end bare soil definition 
    plt_enddate: str, optional
        an override for where you want the plot to end earlier than default
    simple: bool, optional
        whether you want to use simple algoirthm. Default is False
        which means you use azimuth corrected bare soil values
    medfilter: float, optional
        to avoid running daily_avg, you can set median filter in meters;
        this is used to remove large outliers
    ReqTracks: int, optional
        to avoid running daily_avg, you can set required number of tracks 
        to create a daily average RH
    barereq_days: int, optional
        how many bare soil days are required to trust the result, default is 15
    fr : int, optional
        if you want to restrict to a single frequency at the daily-avg stage (1, 20, etc)
    hires_figs :  bool, optional
        whether you want eps instead of png plots

    """
    if (medfilter is not None) and (ReqTracks is not None):
        print('Running daily average')
        txtfile=None; pltit = False
        if fr is not None:
            da.daily_avg(station, medfilter, ReqTracks,  txtfile,pltit,'',2005,2030,fr,False,0,360,False,None)
        else:
            da.daily_avg(station, medfilter, ReqTracks,  txtfile,pltit,'',2005,2030,0,False,0,360,False,None)
        # do not display these plots
        matplt.close ('all')

    # default days of year used for bare soil
    # september from the fall
    doy1 = 244 
    doy2 = 274
    bs = year - 1

    xdir = os.environ['REFL_CODE']
    g.checkFiles(station,'')
    direc = xdir + '/Files/' + station  + '/' 

#   read input file
    if simple:
        gpsfile = direc + station + '_dailyRH.txt'
    else:
        gpsfile = direc + station + '_AllRH.txt'

#   define output files
    outputfile = direc + 'SnowAvg_' + str(year) +'.txt'
    if hires_figs : 
        outputpng = direc + 'water_' + str(year) +'AV.eps'
    else:
        outputpng = direc + 'water_' + str(year) +'AV.png'

    if os.path.exists(gpsfile):
        gps = np.loadtxt(gpsfile,comments='%')
    else:
        print('The input file needed for this code does not exist', gpsfile)
        print('You either need to run daily_avg manually or provide optional inputs here for medfilter and ReqTracks. Exiting')
        sys.exit()

    print('Input file',gpsfile)
    print('Output file: ',outputfile)
    print('Output png: ',outputpng)

    if plt_enddate is not None:
        pyear = int(plt_enddate[0:4])
        pmonth = int(plt_enddate[5:7])
        pday = int(plt_enddate[8:10])
        yend, end_doy = g.cdate2ydoy(plt_enddate)

        end_dt = datetime.datetime(year=pyear, month=pmonth, day = pday)
    else:
        end_doy = None
        end_dt = None


    # this overrides other ways of doing things.
    if bare_date1 is not None:
        bs, doy1 = g.cdate2ydoy(bare_date1)
    if bare_date2 is not None:
        rrrr, doy2 = g.cdate2ydoy(bare_date2)

    if simple:
        sf.snow_simple(station,gps,year,longer, doy1,doy2,bs,plt, end_dt,outputpng,
                outputfile,minS,maxS,barereq_days,end_doy)
    else:
        sf.snow_azimuthal(station,gps,year,longer, doy1,doy2,bs,plt, end_dt,
                outputpng,outputfile,minS,maxS,barereq_days,end_doy)

def main():
    args = parse_arguments()
    snowdepth(**args)


if __name__ == "__main__":
    main()




import datetime
import json
import matplotlib.pyplot as plt
import math
import numpy as np
import os
import scipy.interpolate
import scipy.signal
import subprocess
import sys
import time
import warnings

from importlib.metadata import version

import gnssrefl.gps as g
import gnssrefl.read_snr_files as snr
import gnssrefl.refraction as refr

def gnssir_guts_v2(station,year,doy, snr_type, extension,lsp):
    """

    Computes lomb scargle periodograms for a given station, year, day of year etc.

    Arcs are determined differently than in the first version of the code, which
    was quadrant based. This identifies arcs and applies azimuth constraints after the fact.

    2023-aug-02 trying to fix the issue with azimuth print out being different than
    azimuth at lowest elevation angle

    if screenstats is True, it prints to a log file now, directory $REFL_CODE/logs/ssss

    Parameters
    ----------
    station : str
        4 character station name
    year : int
        full year
    doy : int
        day of year
    snr_type : int
        snr file type
    extension : str
        optional subdirectory to save results

    lsp : dictionary
        e1 : float
            min elev angle, deg
        e2 : float
            max elev angle, deg
        freqs: list of int
            frequencies to use 
        minH : float
            min reflector height, m
        maxH : float
            max reflector height, m 
        NReg : list of floats
            noise region for RH peak2noise , meters
        azval2 : list of floats
            new pairs of azimuth regions, i.e. [0 180 270 360]
        delTmax : float
            max allowed arc length in minutes
        pele: list of floats 
            min and max elev angle in DC removal
        PkNoise : float
            peak to noise value for QC
        ediff : float
            elev angle difference for arc length, QC
        reqAmp : list of floats
            list of required periodogram amplitude for QC for each frequency
        ellist: list of floats
            added 23jun16, allow multiple elevation angle regions
        apriori_rh : float
            a priori reflector height, used in NITE, meters
        
    """

    #   make sure environment variables exist.  set to current directory if not
    g.check_environ_variables()

    # make sure REFL_CODE/Files/station directory exists ... 
    g.checkFiles(station, '')

    if 'ellist' in lsp.keys():
        ellist = lsp['ellist']
        if len(ellist) > 0:
            print('Using an augmented elevation angle list', ellist)
    else:
        ellist = [];
        #print('no augmented elevation angle list')

    # this is also checked in the command line - but for people calling the code ...
    if ((lsp['maxH'] - lsp['minH']) < 5):
        print('Requested reflector heights (', lsp['minH'], ',', lsp['maxH'], ') are too close together. Exiting.')
        print('They must be at least 5 meters apart - and preferably further than that.')
        return

    e1=lsp['e1']; e2=lsp['e2']; minH = lsp['minH']; maxH = lsp['maxH']
    ediff = lsp['ediff']; NReg = lsp['NReg']  
    PkNoise = lsp['PkNoise']; prec = lsp['desiredP']; delTmax = lsp['delTmax']
    if 'azval2' in lsp.keys():
        azval2 = lsp['azval2']; 
        naz = int(len(azval2)/2)
    else:
        print('This module requires azval2 to be set in gnssir_input. This record is not present in your json.')
        sys.exit()

    pele = lsp['pele'] ; pfitV = lsp['polyV']

    freqs = lsp['freqs'] ; reqAmp = lsp['reqAmp'] 

    # allows negatiev az value for first pair
    azvalues = rewrite_azel(azval2)

    ok = g.is_it_legal(freqs)
    if not ok:
        print('There is something wrong. Fix your json list of frequencies. Exiting')
        sys.exit()

    plot_screen = lsp['plt_screen'] 
    onesat = lsp['onesat']; screenstats = lsp['screenstats']
    gzip = lsp['gzip']
    if 'dec' in lsp.keys():
        dec = int(lsp['dec'])
    else:
        dec = 1 # so Jupyter notebooks do not need to be rewritten

    # no need to print to screen if default
    if (dec != 1):
        print('Using decimation value: ', dec)

    d = g.doy2ymd(year,doy); month = d.month; day = d.day
    dmjd, fracS = g.mjd(year,month,day,0,0,0)
    xdir = os.environ['REFL_CODE']
    ann = g.make_nav_dirs(year) # make sure directories are there for orbits
    g.result_directories(station,year,extension) # make directories for the LSP results

   # this defines the minimum number of points in an arc.  This depends entirely on the sampling
    all_lsp = []
   # rate for the receiver, so you should not assume this value is relevant to your case.
    minNumPts = 20
    p,T,irefr,humidity,Tm, lapse_rate = set_refraction_params(station, dmjd, lsp)
    TempK = T + 273.15 # T is in celsius ... I think.
    vapor = humidity
    pressure = p
    temperature = TempK
    # peng used these variables. eventually consolidate
    lat1=lsp['lat']; lon1=lsp['lon']; height1=lsp['ht'] ; mjd1 = dmjd
    lat1R = math.radians(lat1); lon1R = math.radians(lon1); 

    # only used by NITE
    if 'apriori_rh' in lsp.keys():
        RH_apriori = lsp['apriori_rh']  # a priori reflector height used in NITE
    else:
        RH_apriori = 5 # completely made up

    #if screenstats:
    print('Refraction parameters (pressure, temp, humidity, ModelNum)',np.round(p,3),np.round(T,3),np.round(humidity,3),irefr)

    if (irefr >= 3) & (irefr <= 6):
        # N_ant is the index of refraction
        # apparently this is wrong ?
        #N_ant=refr.refrc_Rueger(p,humidity,TempK)[0]    #
        N_ant = refr.refrc_Rueger(pressure-vapor, vapor, TempK)[0]   #get antenne refractivity
        #ztd=2.3                                             #zenith total delay from PPP
        # hydrostatic delay, Saastamoinen
        zhd = refr.saastam2(pressure, lat1, height1)             
        # wet delay 
        zwd = refr.asknewet(humidity, Tm, lapse_rate)
        print('Dry and wet zenith delays, meters ', round(zhd,3),round(zwd,3))
        # if you had estimated total delay, you could get wet delay by 
        ztd = zhd + zwd 

# only doing one day at a time for now - but have started defining the needed inputs for using it
    twoDays = False
    obsfile2= '' # dummy value for name of file for the day before, when we get to that
    fname, resultExist = g.LSPresult_name(station,year,doy,extension) 
    print('Results are written to:', fname)
    if screenstats:
        logid, logfilename = open_gnssir_logfile(station,year,doy,extension)
    else:
        logid = None 
        logfilename = None


    if (lsp['nooverwrite'] == True) & (resultExist == True):
        allGood = 0
        print('>>>>> The result file already exists for this day and you have selected the do not overwrite option')
        #sys.exit()
    else:
        # uncompress here so you should not have to do it in read_snr_multiday ...
        obsfile, obsfileCmp, snre = g.define_and_xz_snr(station,year,doy,snr_type) 

        allGood, snrD, nrows, ncols = read_snr(obsfile)
        # added gzip option.  first input is xz compression
        if allGood and (dec != 1):
            print('Invoking decimation option')
            # does dec need to be a list??? was in original code
            # get the indices
            rem_arr = np.remainder( snrD[:,3], dec)
            # find out where they are zero
            iss =  (rem_arr == 0)
            # and apply
            snrD = snrD[iss,:]
            # not sure nrows and ncols is being used ... so not redoing it


        snr.compress_snr_files(lsp['wantCompression'], obsfile, obsfile2,twoDays,gzip) 
    if (allGood == 1):
        print('Reading SNR data from: ', obsfile)
        #print('Results will be written to:', fname)
        minObsE = min(snrD[:,1])
        print('Min observed elev. angle ', station, year, doy, minObsE, '/requested e1 and e2 ', e1,e2)
        # only apply this test for simple e1 and e2
        if len(ellist) == 0:
            if minObsE > (e1 + ediff):
                print('You literally have no data above the minimum elevation angle setting')
                print('which is e1 + ediff: ', e1 + ediff, ' If you do not like')
                print('this QC constraint, then set ediff to be very large (10 degrees) in the json or use ')
                print('the minimum elevation angle your receiver used. Exiting.')
                sys.exit()

        if (irefr == 3) or (irefr == 4):
            # elev refraction, lsp, pressure, temperature- units?, time, sat
            if irefr == 3:
                print('Ulich refraction correction')
            else:
                print('Ulich refraction correction, time-varying')
            # I do not understand why all these extra parameters are sent to this 
            # function as they are not used. Maybe I was doing some testing.
            ele=refr.Ulich_Bending_Angle(snrD[:,1],N_ant,lsp,p,T,snrD[:,3],snrD[:,0])
        elif (irefr == 5 ) or (irefr == 6):
            # NITE MODEL
            # remove ele < 1.5 cause it blows up
            i=snrD[:,1] > 1.5
            snrD = snrD[i,:]
            N = len(snrD[:,1])
            # elevation angles, degrees
            ele=snrD[:,1]
            # zenith angle in radians
            zenithA = 0.5*np.pi - np.radians(ele)
            #get mapping function and derivatives
            [gmf_h,dgmf_h,gmf_w,dgmf_w]=refr.gmf_deriv(mjd1, lat1R, lon1R,height1,zenithA)
            [mpf, dmpf]=[refr.mpf_tot(gmf_h,gmf_w,zhd,zwd),refr.mpf_tot(dgmf_h,dgmf_w,zhd,zwd)]

            # inputs apriori RH, elevation angle, refractive index, zenith delay, mpf ?, dmpf?
            if irefr == 5:
                print('NITE refraction correction, Peng et al. remove data < 1.5 degrees')
                dE_NITE=refr.Equivalent_Angle_Corr_NITE(RH_apriori, ele, N_ant, ztd, mpf, dmpf)
                ele = ele + dE_NITE 
            else: 
                print('MPF refraction correction, Wiliams and Nievinski')
                dE_MPF= refr.Equivalent_Angle_Corr_mpf(ele,mpf,N_ant,RH_apriori)
                ele = ele + dE_MPF 

        elif irefr == 0:
            print('No refraction correction applied ')
            ele = snrD[:,1]
        else :
            if irefr == 1:
                if screenstats:
                    print('Standard Bennett refraction correction')
            else:
                if screenstats:
                    print('Standard Bennett refraction correction, time-varying')
            ele = snrD[:,1]
            ele=apply_refraction_corr(lsp,ele,p,T)

        # apply an elevation mask for all the data for the polynomial fit
        i= (ele >= pele[0]) & (ele < pele[1])
        ele = ele[i]
        snrD = snrD[i,:]
        # why doesn't this have an i index, is it not used?
        sats = snrD[:,0]
        # make sure the snrD array has elevation angles fixed
        snrD[:,1] = ele # ????

        # open output file
        fout,frej = g.open_outputfile(station,year,doy,extension) 
#  main loop a given list of frequencies
        total_arcs = 0
        ct = 0
        for f in freqs:
            found_results = False
            if plot_screen: 
                # no idea if this will work
                fig, (ax1, ax2) = plt.subplots(2, 1,figsize=(10,7))
            rj = 0
            gj = 0
            if screenstats: 
                logid.write('=================================================================================\n')
                logid.write('Looking at {0:4s} {1:4.0f} {2:3.0f} frequency {3:3.0f} ReqAmp {4:7.2f} \n'.format(station, year, doy,f,reqAmp[ct]))
                logid.write('=================================================================================\n')
                print('**** looking at frequency ', f, ' ReqAmp', reqAmp[ct], ' doy ', doy, 'ymd', year, month, day )
#           get the list of satellites for this frequency
            if onesat == None:
                satlist = find_mgnss_satlist(f,year,doy)
            else:
                # check that your requested satellite is the right frequency
                satlist = onesat_freq_check(onesat,f )

            # main satellite loop
            for satNu in satlist:
                if screenstats: 
                    print('Satellite', satNu)
                    logid.write('satellite {0:3.0f} \n'.format(satNu))

                iii = (sats == satNu)
                thissat = snrD[iii,:]
                goahead = False

                if len(thissat) > 0:
                    # if there are data for this satellite, find all the arcs
                    # separate numpy array of elevation angles .... 

                    # allow more than one set of elevation angles
                    if len(ellist) > 0:
                        arclist = np.empty(shape=[0,6])
                        for ij in range(0,len(ellist),2):
                            te1 = ellist[ij]; te2 = ellist[ij+1]; newl = [te1,te2]
                            # poorly named inputs - elev, azimuth, seconds of the day, ...
                            # te1 and te2 are the requested elevation angles I believe
                            # satNu is the requested satellite number
                            tarclist = new_rise_set_again(thissat[:,1],thissat[:,2],thissat[:,3],te1, te2,ediff,satNu,screenstats,logid)
                            arclist = np.append(arclist, tarclist,axis=0)

                    else:
                        arclist = new_rise_set_again(thissat[:,1],thissat[:,2],thissat[:,3],e1, e2,ediff,satNu,screenstats,logid)

                    nr,nc = arclist.shape
                    if nr > 0:
                        goahead = True
                    else:
                        goahead = False

                if goahead:
                    found_results = True
                    # instead of az bins now go through each arc 
                    for a in range(0,nr):
                        sind = int(arclist[a,0]) ; eind = int(arclist[a,1])
                        #if screenstats:
                        #    print('Indices for this arc', a, sind, eind)
                        # create array for the requested arc
                        d2 = np.array(thissat[sind:eind, :], dtype=float)
                        # window the data - which also removes DC 
                        # this is saying that these are the min and max elev angles you should be using
                        e1 = arclist[a,4]; e2 = arclist[a,5]
                        x,y, Nvv, cf, meanTime,avgAzim,outFact1, Edot2, delT= window_new(d2, f, 
                                satNu,ncols,pele, lsp['polyV'],e1,e2,azvalues,screenstats)
                        Nv = Nvv # number of points
                        UTCtime = meanTime

                        if (delT != 0):
                            MJD = g.getMJD(year,month,day, meanTime)
                            maxF, maxAmp, eminObs, emaxObs,riseSet,px,pz= g.strip_compute(x,y,cf,maxH,prec,pfitV,minH)

                            tooclose = False
                            if (maxF ==0) & (maxAmp == 0):
                                tooclose == True; Noise = 1; iAzim = 0;
                            else:
                                nij =  pz[(px > NReg[0]) & (px < NReg[1])]

                            Noise = 1
                            if (len(nij) > 0):
                                Noise = np.mean(nij)

                            iAzim = int(avgAzim)

                            if abs(maxF - minH) < 0.10: #  peak too close to min value
                                tooclose = True

                            if abs(maxF - maxH) < 0.10: #  peak too close to max value
                                tooclose = True

                            if False:
                                print(avgAzim, satNu, e1, e2)

                            if (not tooclose) & (delT < delTmax) & (maxAmp > reqAmp[ct]) & (maxAmp/Noise > PkNoise):
                            # request from a tide gauge person for Month, Day, Hour, Minute

                                if lsp['mmdd']:
                                    ctime = g.nicerTime(UTCtime); ctime2 = ctime[0:2] + ' ' + ctime[3:5]
                                    fout.write(" {0:4.0f} {1:3.0f} {2:6.3f} {3:3.0f} {4:6.3f} {5:6.2f} {6:6.2f} {7:6.2f} {8:6.2f} {9:4.0f} {10:3.0f} {11:2.0f} {12:8.5f} {13:6.2f} {14:7.2f} {15:12.6f} {16:1.0f} {17:2.0f} {18:2.0f} {19:5s} \n".format(year,doy,maxF,satNu, UTCtime, avgAzim,maxAmp,eminObs,emaxObs,Nv, f,riseSet, Edot2, maxAmp/Noise, delT, MJD,irefr,month,day,ctime2)) 
                                    onelsp = [year,doy,maxF,satNu,UTCtime,avgAzim,maxAmp, eminObs,emaxObs,Nv,f,riseSet,Edot2,maxAmp/Noise,delT,MJD,irefr,month,day,ctime2]

                                else:
                                    onelsp = [year,doy,maxF,satNu,UTCtime,avgAzim,maxAmp, eminObs,emaxObs,Nv,f,riseSet,Edot2,maxAmp/Noise,delT,MJD,irefr]
                                    fout.write(" {0:4.0f} {1:3.0f} {2:6.3f} {3:3.0f} {4:6.3f} {5:6.2f} {6:6.2f} {7:6.2f} {8:6.2f} {9:4.0f} {10:3.0f} {11:2.0f} {12:8.5f} {13:6.2f} {14:7.2f} {15:12.6f} {16:1.0f} \n".format(year,doy,maxF,satNu, UTCtime, avgAzim,maxAmp,eminObs,emaxObs,Nv, f,riseSet, Edot2, maxAmp/Noise, delT, MJD,irefr)) 
                                gj +=1

                                all_lsp.append(onelsp)
                                if screenstats:
                                    T = g.nicerTime(UTCtime)
                                    logid.write('SUCCESS Azimuth {0:3.0f} Sat {1:3.0f} RH {2:7.3f} m PkNoise {3:4.1f} Amp {4:4.1f} Fr{5:3.0f} UTC {6:5s} DT {7:3.0f} \n'.format(iAzim,satNu,maxF,maxAmp/Noise,maxAmp, f,T,round(delT)))
                                    print('SUCCESS Azimuth {0:3.0f} Sat {1:3.0f} RH {2:7.3f} m PkNoise {3:4.1f} Amp {4:4.1f} Fr{5:3.0f} UTC {6:5s} DT {7:3.0f} '.format(iAzim,satNu,maxF,maxAmp/Noise,maxAmp, f,T,round(delT)))
                                if plot_screen:
                                    failed = False
                                    local_update_plot(x,y,px,pz,ax1,ax2,failed)
                            else:
                                rj +=1
                                if screenstats:
                                    #print(delT, tooclose,Noise,PkNoise)
                                    #35.0 False 3.682862189099345 2.8

                                    logid.write('FAILED QC for Azimuth {0:.1f} Satellite {1:2.0f} UTC {2:5.2f} RH {3:5.2f} \n'.format( iAzim,satNu,UTCtime,maxF))
                                    print('FAILED QC for Azimuth {0:.1f} Satellite {1:2.0f} UTC {2:5.2f} RH {3:5.2f}'.format( iAzim,satNu,UTCtime,maxF))
                                    g.write_QC_fails(delT,lsp['delTmax'],eminObs,emaxObs,e1,e2,ediff,maxAmp, Noise,PkNoise,reqAmp[ct],tooclose,logid)
                                if plot_screen:
                                    failed = True
                                    local_update_plot(x,y,px,pz,ax1,ax2,failed)

            if screenstats:
                print('=================================================================================')
                print('     Frequency ', f, ' good arcs:', gj, ' rejected arcs:', rj )
                print('=================================================================================')
                logid.write('=================================================================================\n')
                logid.write('     Frequency  {0:3.0f}   good arcs: {1:3.0f}  rejected arcs: {2:3.0f} \n'.format( f, gj, rj))
                logid.write('=================================================================================\n')
            total_arcs = gj + total_arcs
# close the output files
            ct += 1
            if found_results and plot_screen:
                print('data found for this frequency: ',f)
                ax1.set_xlabel('Elevation Angles (deg)')
                ax1.grid(True, linestyle='-'); ax2.grid(True, linestyle='-')
                ax1.set_title(station + ' Raw Data/Periodogram for ' + g.ftitle(f) + ' Frequency')
                ax2.set_xlabel('Reflector Height (m)');
                ax2.set_ylabel('volts/volts') ; ax1.set_ylabel('volts/volts')
                plotname = xdir + '/Files/' + station + '/gnssir_freq' + str(f) + '.png'
                g.save_plot(plotname)
                plt.show()
                #plot2screen(station, f, ax1, ax2,lsp['pltname']) 
            else:
                if plot_screen: 
                    print('no data found for this frequency: ',f)
                    #plt.close()

        fout.close() ; # these are the LSP results written to text file 
        # try moving this
        if found_results and plot_screen:
            plot2screen(station, f, ax1, ax2,lsp['pltname']) 

        if screenstats:
            logid.close()
            print('Screen stat information printed to: ', logfilename)

        head = g.lsp_header(station) # header
        # convert to numpy array
        allL = np.asarray(all_lsp)
        # sort the results for felipe
        ii = np.argsort(allL[:,15])
        allL = allL[ii,:]

        if lsp['mmdd']:
            f = '%4.0f %3.0f %6.3f %3.0f %6.3f %6.2f %6.2f %6.2f %6.2f %4.0f  %3.0f  %2.0f %8.5f %6.2f %7.2f %12.6f %1.0f %2.0f %2.0f %5s'
        else:
            f = '%4.0f %3.0f %6.3f %3.0f %6.3f %6.2f %6.2f %6.2f %6.2f %4.0f  %3.0f  %2.0f %8.5f %6.2f %7.2f %12.6f %1.0f'

        # this is really just overwriting what I had before. However, This will be sorted.
        testfile,fe = g.LSPresult_name(station,year,doy,extension)
        print('Writing sorted LSP results to : ', testfile, '\n')
        np.savetxt(testfile, allL, fmt=f, delimiter=' ', newline='\n',header=head, comments='%')

def set_refraction_params(station, dmjd,lsp):
    """
    set values used in refraction correction

    Parameters
    ----------
    station : str
        4 character station name

    dmjd : float
        modified julian date

    lsp : dictionary with information about the station
        lat : float
            latitude, deg
        lon : float
            longitude, deg
        ht : float
            height, ellipsoidal

    Returns
    -------
    p : float
        pressure, hPa
    T : float
        temperature, Celsius
    irefr : int
        refraction model number I believe, which is also sent, so not needed
    e : float
        water vapor pressure, hPa
    Tm : float
        temperature in Kelvin
    la : float
        lapse rate

    """

    if 'refr_model' not in lsp.keys():
        # default is always to use refraction model 1
        # if for some reason it is missing ...
        refraction_model = 1
    else:
        refraction_model = lsp['refr_model']


    xdir = os.environ['REFL_CODE']
    # default values
    p = 0; T = 0; irefr = 0; e=0
    if lsp['refraction']:
        irefr = refraction_model
        refr.readWrite_gpt2_1w(xdir, station, lsp['lat'], lsp['lon'])
        # time varying was originally set to no for now (it = 1)
        # now allows time varying for models 2, 4 and now MPF and NITE
        if refraction_model in [2, 4, 5, 6]:
            it = 0
            print('Using time-varying refraction parameter inputs')
        #elif (refraction_model == 5):
        else:
            it = 1
        dlat = lsp['lat']*np.pi/180; dlong = lsp['lon']*np.pi/180; ht = lsp['ht']
        p,T,dT,Tm,e,ah,aw,la,undu = refr.gpt2_1w(station, dmjd,dlat,dlong,ht,it)
        # e is water vapor pressure, so humidity ??
        #print("Pressure {0:8.2f} Temperature {1:6.1f} \n".format(p,T))

    #print('refraction model', refraction_model,irefr)
    return p,T,irefr, e, Tm, la

def apply_refraction_corr(lsp,ele,p,T):
    """

    Parameters
    ----------
    lsp : dictionary
        info from make_json_input such as station lat and lon
    ele : numpy array of floats
        elevation angles  (deg)
    p : float
        pressure
    T : float
        temperature (C)

    Returns
    -------
    ele : numpy array of floats
         elevation angle (deg)
    """
    if lsp['refraction']:
        #print('<<<<<< apply refraction correction >>>>>>')
        corrE = refr.corr_el_angles(ele, p,T)
        ele = corrE

    return ele

def local_update_plot(x,y,px,pz,ax1, ax2,failure):
    """
    updates optional result plot for SNR data and Lomb Scargle periodograms

    Parameters
    ----------
    x : numpy array
        elevation angle (deg)
    y : numpy array
        SNR (volt/volt)
    px : numpy array
        reflector height (m)
    pz : numpy array
        spectral amplitude (volt/volt)
    ax1 : matplotlib figure control
        top plot
    ax2 : matplotlib figure control
        bottom plot
    failure : boolean
        whether periodogram fails QC 

    """
    if failure:
        ax1.plot(x,y,color='gray',linewidth=0.5)
        ax2.plot(px,pz,color='gray',linewidth=0.5)
    else:
        ax1.plot(x,y)
        ax2.plot(px,pz)


def plot2screen(station, f,ax1,ax2,pltname):
    """
    Add axis information and Send the plot to the screen.
    https://www.semicolonworld.com/question/57658/matplotlib-adding-an-axes-using-the-same-arguments-as-a-previous-axes

    Parameters
    ----------
    station : string
        4 character station ID

    """
    ax2.set_xlabel('Reflector Height (m)'); 
    #ax2.set_title('SNR periodogram')
    ax2.set_ylabel('volts/volts')
    ax1.set_ylabel('volts/volts')
    ax1.set_xlabel('Elevation Angles (deg)')
    ax1.grid(True, linestyle='-')
    ax2.grid(True, linestyle='-')
    ax1.set_title(station + ' SNR Data/' + g.ftitle(f) + ' Frequency')
    plt.show()

    return True


def read_json_file(station, extension,**kwargs):
    """
    picks up json instructions for calculation of lomb scargle periodogram

    Parameters
    ----------
    station : str
        4 character station name

    extension : str
        experimental subdirectory - default is ''

    Returns
    -------
    lsp : dictionary

    """
    lsp = {} # 
    # pick up a new parameter - that will be True for people looking
    # for the json but that aren't upset it does not exist.
    noexit = kwargs.get('noexit',False)
    # leftover from when i was using None
    if len(extension) == 0:
        useextension = False
        instructions_ext = ''
    else:
        useextension = True 
        instructions_ext = str(os.environ['REFL_CODE']) + '/input/' + station + '.' + extension + '.json'

    instructions = str(os.environ['REFL_CODE']) + '/input/' + station + '.json'

    if useextension and os.path.isfile(instructions_ext):
        usefile = instructions_ext
        print('Using these instructions ', usefile)
        with open(instructions_ext) as f:
            lsp = json.load(f)
    else:
        usefile = instructions
        if os.path.isfile(instructions):
            print('Using these instructions ', usefile)
            with open(instructions) as f:
                lsp = json.load(f)
        else:
            if noexit:
                lsp = {}
                return lsp
            else:
                print('The json instruction file does not exist: ', instructions)
                print('Please make with gnssir_input and run this code again.')
                sys.exit()

    if len(lsp['reqAmp']) < len(lsp['freqs']) :
        print('Number of frequencies found in json: ', len(lsp['freqs']))
        print('Number of required amplitudes found in json: ', len(lsp['reqAmp']))
        print('You need to have a required Amplitude for each frequency.')
        print('Please fix your json file: ', usefile)
        sys.exit()

    return lsp


def onesat_freq_check(satlist,f ):
    """
    for a given satellite name - tries to determine
    if you have a compatible frequency

    Parameters
    ----------
    satlist : list 
        integer
    f : integer
        frequency

    Returns
    -------
    satlist : list
        integer 
    """
    isat = int(satlist[0])
    if (isat < 100):
        if (f > 100):
            print('wrong satellite name for this frequency:', f)
            satlist = [] # ???
    elif (isat >= 101) & (isat < 200):
        if (f < 101) | (f > 102):
            print('wrong satellite name for this frequency:', f)
            satlist = [] # ???
    elif (isat > 201) & (isat < 300):
        if (f < 201) | (f > 208):
            print('wrong satellite name for this frequency:', f)
            satlist = [] # ???
    elif (isat > 300) & (isat < 400):
        if (f < 302) | (f > 307):
            print('wrong satellite name for this frequency:', f)
            satlist = [] # ???
    else:
        satlist= []

    return satlist


def new_rise_set(elv,azm,dates, e1, e2, ediff,sat, screenstats):
    """
    This provides a list of rising and setting arcs 
    for a given satellite in a SNR file
    based on using changes in elevation angle

    I think this is used in quickLook but not gnssir

    Parameters
    ----------
    elv : numpy array  of floats
        elevation angles from SNR file
    azm : numpy array  of floats
        azimuth angles from SNR file
    dates : numpy array  of floats
        seconds of the day from SNR file
    e1 : float
        min eval
    e2 : float
        max eval
    ediff : float
        el angle difference QC
    sat : int
        satellite number

    Returns
    -------
    tv : numpy array
        beginning and ending indices of the arc
        satellite number, arc number

    """
    # require arcs to be this length in elev angle
    min_deg = (e2-ediff)   - (e1 + ediff)

#   time limit in seconds - taken from david
    gaptlim = 5*60 # seems awfully small
    #newf = np.array(f[i, :], dtype=float)
    iarc = 0
    ddate = np.ediff1d(dates)
    delv = np.ediff1d(elv)
    bkpt = len(ddate)
    bkpt = np.append(bkpt, np.where(ddate > gaptlim)[0])
    bkpt = np.append(bkpt,  np.where(np.diff(np.sign(delv)))[0])
    bkpt = np.unique(bkpt)
    bkpt = np.sort(bkpt)
    N=len(bkpt)
    tv = np.empty(shape=[0,4])

    for ii in range(N):
        if ii == 0:
            sind = 0
        else:
            sind = bkpt[ii - 1] + 1
        eind = bkpt[ii] + 1
        nelv = np.array(elv[sind:eind], dtype=float)
        nazm = np.array(azm[sind:eind], dtype=float)
        nt = np.array(dates[sind:eind], dtype=float)
        #nazm = azm[sind:eind]
        minObse = min(nelv)
        maxObse = max(nelv)
        minA = min(nazm)
        maxA = max(nazm)

        nogood = False
        verysmall = False
        ediff_violation = False
        if (minObse - e1) > ediff:
            nogood = True
            ediff_violation = True 
        if (maxObse - e2) < -ediff:
            nogood = True
            ediff_violation = True
        if (eind-sind) == 1:
            nogood = True
            verysmall = True
        if ((maxObse - minObse) < min_deg):
            nogood = True

        if screenstats:
            if nogood:
                # do not write out warning for these tiny arcs which should not even be there.
                # i am likely reading the code incorrectly
                add = ''
                if ediff_violation:
                    add = ' violates ediff'
                if not verysmall:
                    print('Failed sat/arc',sat,iarc+1, sind,eind,' min/max elev: ', np.round(minObse,2), np.round(maxObse,2), minA,maxA,add)
            else:
                print('Keep   sat/arc',sat,iarc+1, sind,eind,' min/max elev: ', np.round(minObse,2), np.round(maxObse,2),minA,maxA)

        if not nogood :
            iarc = iarc + 1
            newl = [sind, eind, int(sat), iarc]
            tv = np.append(tv, [newl],axis=0)

    return tv 

def read_snr(obsfile):
    """
    Simple function to load the contents of a SNR file into a numpy array

    Parameters
    ----------
    obsfile : str
        name of the snrfile 

    Returns
    -------
    allGood : int 
        1, file was successfully loaded, 0 if not.
        apparently this variable was defined when I did not know about booleans....
    f : numpy array
        contents of the SNR file
    r : int
        number of rows in SNR file
    c : int
        number of columns in SNR file

    """
    allGood = 1
    if os.path.isfile(obsfile):
        f = np.genfromtxt(obsfile,comments='%')
    else:
        print('No SNR file found')
        allGood = 0
        return allGood, 0, 0, 0
    r,c = f.shape
    if (r > 0) & (c > 0):
        i= f[:,1] > 0
        f=f[i,:]
    if r == 0:
        print('No rows in this file!')
        allGood = 0
    if c == 0:
        print('No columns in this file!')
        allGood = 0 

    return allGood, f, r, c


def window_new(snrD, f, satNu,ncols,pele,pfitV,e1,e2,azlist,screenstats):
    """
    retrieves SNR arcs for a given satellite. returns elevation angle and 
    detrended linear SNR

    2023-aug02 updated to improve azimuth calculation reported

    Parameters
    ----------
    snrD : numpy array (multiD)
        contents of the snr file, i.e. 0 column is satellite numbers, 1 column elevation angle ...
    f : int
        frequency you want
    satNu : int
        requested satellite number
    ncols : int
        how many columns does the SNR file have
    pele : list of floats
        elevation angles for polynomial fit
    pfitV : float
        polynomial order
    e1 : float
        requested min elev angle (deg)
    e2 : float
        requested max elev angle (deg)
    azlist : list of floats (deg)
        non-continguous azimuth regions, corrected for negative regions
    screenstats : bool
        whether you want debugging information

    Returns
    -------
    x : numpy array of floats
        elevation angle, degrees
    y : numpy array of floats
        linear SNR with DC removed
    Nvv :  int
        number of points in x/y array
    cf : float
        scale factor for requested frequency (used in LSP)
    meanTime : float
        UTC hour of the day (GPS time)
    avgAzim : float
        average azimuth of the arc (deg) 
        ### this will not be entirely consistent with other metric
    outFact1: float
        kept for backwards compatibility.  set to zero
    outFact2 : float
        edot factor used in RH dot correction
    delT : float
        arc length in minutes

    """
    x=[]; y=[]; azi=[]; seconds = []; edot = [] ; sat = []
    Nvv= 0; meanTime = 0; avgAzim = 0 ; outFact2 = 0 ; delT = 0
    initA = 0;
    cf = g.arc_scaleF(f,satNu)
    outFact1 = 0 # backwards compatability
    good = True

    if f in [1,101,201,301]:
        column = 7
    elif f in [2,20,102,302]:
        column = 8
    elif f in [5, 205]:
        column = 9
#   these are galileo frequencies (via RINEX definition)
    elif f in [206,306]:
        column = 6
    elif f in [207,307]:
        column = 10
    elif (f == 208):
        column = 11
    else:
        column = 0
        if screenstats:
            print('You asked for a frequency I do not recognize: ', f)
        good = False
    #python column
    icol = column - 1
    #print('using frequency ', f, ' columns ', ncols)


    if (column <= ncols) : 
        # at least there is a column where there should be
        # sep 26, 2023
        # these definitions used to be outside the if, but putting them inside now
        datatest = snrD[:,icol]
        ijk = (datatest == 0)
        nzero = len(datatest[ijk])
        if np.sum(datatest) < 1:
            if screenstats:
                print('No useful data on frequency ', f , 'and sat ', satNu, ': all zeros')
            good = False
        else:
            if nzero > 0:
                #print('removing ', nzero, ' zero points on frequency ', f )
                # indices you are keeping ... 
                nn = (datatest > 0) ; 
                snrD = snrD[nn,:]

        sat = snrD[:,0]
        ele = snrD[:,1]
        azm = snrD[:,2]
        seconds = snrD[:,3]
        edot = snrD[:,4] # but does not have to exist?
        data = snrD[:,icol]

        # not really good - but at least the zeros have been removed
        if good:
            # change to linear units
            data = np.power(10,(data/20))
            if len(ele) > 20:
                model = np.polyfit(ele,data,pfitV)
                fit = np.polyval(model,ele)
                data = data - fit
                # apply elevation angle constraint
                i =  (ele > e1) & (ele <= e2)
                # arbitrary
                Nvv = len(ele[i])
                if Nvv > 15:
                    # get the index of the min elevation angle
                    ie = np.argmin(ele[i])
                    # find the azimuth of that first elevation angle 
                    initA = azm[i][ie]
                    #print('initial azimuth ostensibly for min eangle',initA)
                    keeparc = check_azim_compliance(initA,azlist)
                    if keeparc :
                        x = ele[i] ; y = data[i]
                        edot = edot[i]
                        sat = sat[i] ; azm = azm[i]
                        seconds = seconds[i]
                    else:
                        if screenstats:
                            print('This azimuth is not in the azimuth region', initA, '(deg)')
                        good = False
    else:
        if screenstats:
            print('Asked for data that are not there.')
        good = False
    
    # now calculate edot factor
    if good:
        if len(x) > 0:
            #Taylor Smith - Chasing down error:
            #...site-packages/gnssrefl/gnssir_v2.py", line 741, in window_new
                #model = np.polyfit(seconds,x*np.pi/180,1)
                #TypeError: can't multiply sequence by non-int of type 'float'
            #NOTE: FAILS ON EMPTY X -- Breaks any loops
    
            #sec_float = [float(x) for x in seconds]
            #model = np.polyfit(sec_float,x*np.pi/180,1)
        
            #dd = np.diff(seconds)
            # edot, in radians/sec
            model = np.polyfit(seconds,x*np.pi/180,1)
        
            avgEdot_fit = model[0]
            avgAzim = np.mean(azm)
            meanTime = np.mean(seconds)/3600
    #  delta Time in minutes
            delT = (np.max(seconds) - np.min(seconds))/60
    # average tan(elev)
            cunit = np.mean(np.tan(np.pi*x/180))
    #       return tan(e)/edot, in units of one over (radians/hour) now. used for RHdot correction
    #       so when multiplyed by RHdot - which would be meters/hours ===>>> you will get a meter correction
    
            outFact2 = cunit/(avgEdot_fit*3600)

    # This originally returned the average azimuth.
    # I think you can return the initA instead of avgAzim here
    #    return x,y, Nvv, cf, meanTime,avgAzim,outFact1, outFact2, delT
    #print('new ', initA, ' old ', avgAzim)
    return x,y, Nvv, cf, meanTime,initA, outFact1, outFact2, delT


def find_mgnss_satlist(f,year,doy):
    """
    find satellite list for a given frequency and date

    Parameters
    ----------
    f : integer
        frequency
    snrExist : numpy array, bool
        tells you if a signal is (potentially) legal
    year : int
        full year
    doy : int
        day of year

    Returns
    -------
    satlist: numpy list of integers
        satellites to use

    """
    # get list of relevant satellites
    l2c_sat, l5_sat = g.l2c_l5_list(year,doy)

    l1_sat = np.arange(1,33,1)
    satlist = []
    if f == 1:
        satlist = l1_sat
    if f == 20:
        satlist = l2c_sat
    if f == 2:
        satlist = l1_sat
    if f == 5:
        satlist = l5_sat
# only have 24 frequencies defined for glonass
    if (f == 101) or (f==102):
        satlist = np.arange(101,125,1)
#   galileo - 40 max?
    gfs = int(f-200)

    if (f >  200) and (f < 210):
        satlist = np.arange(201,241,1)
#   galileo has no L2 frequency, so set that always to zero
    if f == 202:
        satlist = []
#   pretend there are 32 Beidou satellites for now
    if (f > 300):
        satlist = np.arange(301,333,1)

    return satlist



def rewrite_azel(azval2):
    """
    Trying to allow regions that cross zero degrees azimuth

    Parameters
    ----------
    azval2 : list of floats
        input azimuth regions

    Returns
    -------
    azelout : list of floats
        azimuth regions without negative numbers ... 

    """

    # if nothing changes
    azelout = azval2
    #print('Requested azimuths: ', azval2)

    # check for negative beginning azimuths
    N2 = len(azval2) ; a1 = int(azval2[0]); a2 = int(azval2[1])

    if (N2 % 2) != 0:
        print('Azimuth regions must be in pairs. Please check the azval2 variable in your json input file')
        sys.exit()
    if (N2 == 2) & (a1 < 0):
        azelout = [a1+360, 360, 0,  a2]
    if (N2 == 4) & (a1 < 0):
        azelout = [a1+360, 360, 0,  a2, int(azval2[2]), int(azval2[3])]
    if (N2 == 6) & (a1 < 0):
        azelout = [a1+360, 360, 0,  a2, int(azval2[2]), int(azval2[3]), int(azval2[4]), int(azval2[5])]
    if (N2 == 8) & (a1 < 0):
        azelout = [a1+360, 360, 0,  a2, azval2[2], azval2[3], azval2[4], azval2[5], azval2[6], azval2[7]]
    if N2 > 8:
        print('Not going to allow more than four azimuth regions ...')
        sys.exit()

    #print('Using azimuths: ', azelout)
    return azelout

def check_azim_compliance(initA,azlist):
    """
    Check to see if your arc is in one of the requested regions

    Parameters
    ----------
    initA : float
        azimuth of selected arc (deg)
    azlist : list of floats
        list of acceptable azimuth regions

    Returns
    -------
    keeparc : bool
        whether the arc is in a selected azimuth range
    """
    keeparc = False
    N = int(len(azlist)/2)
    for a in range(0,N):
        azim1 = azlist[2*a]; azim2 = azlist[2*a+1]
        if (initA>= azim1) & (initA <= azim2):
            keeparc = True
            #print('found one in requested region', azim1, azim2)

    return keeparc

def new_rise_set_again(elv,azm,dates, e1, e2, ediff,sat, screenstats,logid ):
    """
    This provides a list of rising and setting arcs 
    for a given satellite in a SNR file
    based on using changes in elevation angle

    Parameters
    ----------
    elv : numpy array  of floats
        elevation angles from SNR file
    azm : numpy array  of floats
        azimuth angles from SNR file
    dates : numpy array  of floats
        seconds of the day from SNR file
    e1 : float
        min elevation angle (deg)
    e2 : float
        max elevation angle (deg)
    ediff : float
        el angle difference required, deg, QC
    sat : int
        satellite number
    screenstats : bool
        whether you want info printed to the screen
    logid: fileid
        where the screen stat info is stored as a file

    Returns
    -------
    tv : numpy array
        beginning and ending indices of the arc
        satellite number, arc number, elev min, elev max

    """
    # require arcs to be this length in elev angle
    min_deg = (e2-ediff)   - (e1 + ediff)

#   time limit in seconds - taken from david purnell
    gaptlim = 5*60 # seems awfully small
    #newf = np.array(f[i, :], dtype=float)
    iarc = 0
    ddate = np.ediff1d(dates)
    delv = np.ediff1d(elv)
    bkpt = len(ddate)
    bkpt = np.append(bkpt, np.where(ddate > gaptlim)[0])
    bkpt = np.append(bkpt,  np.where(np.diff(np.sign(delv)))[0])
    bkpt = np.unique(bkpt)
    bkpt = np.sort(bkpt)
    N=len(bkpt)
    tv = np.empty(shape=[0,6])

    for ii in range(N):
        if ii == 0:
            sind = 0
        else:
            sind = bkpt[ii - 1] + 1
        eind = bkpt[ii] + 1
        nelv = np.array(elv[sind:eind], dtype=float)
        nazm = np.array(azm[sind:eind], dtype=float)
        nt = np.array(dates[sind:eind], dtype=float)
        #nazm = azm[sind:eind]
        minObse = min(nelv)
        maxObse = max(nelv)
        #print('min/max obs e ', minObse, maxObse)
        # how to get the azimuth?
        minA = min(nazm)
        maxA = max(nazm)


        nogood = False
        verysmall = False
        ediff_violation = False
        if (minObse - e1) > ediff:
            nogood = True
            ediff_violation = True 
        if (maxObse - e2) < -ediff:
            nogood = True
            ediff_violation = True
        if (eind-sind) == 1:
            nogood = True
            verysmall = True
        if ((maxObse - minObse) < min_deg):
            nogood = True

        if screenstats:
            if nogood:
                # do not write out warning for these tiny arcs which should not even be there.
                # i am likely reading the code incorrectly
                add = ''
                if ediff_violation:
                    add = ' violates ediff'
                if not verysmall:
                    print('Failed sat/arc',sat,iarc+1, sind,eind,' min/max elev: ', np.round(minObse,2), np.round(maxObse,2), minA,maxA,add)
                    logid.write('Failed sat/arc {0:3.0f} {1:3.0f} {2:7.2f} index {3:7.2f} min/max elev: {4:7.2f} {5:7.2f} Azims: {6:6.2f} {7:6.2f} {8:15s}  \n'.format( sat,iarc+1,sind,eind,np.round(minObse,2), np.round(maxObse,2), minA,maxA,add))
            else:
                print('Keep   sat/arc',sat,iarc+1, sind,eind,' min/max elev: ', np.round(minObse,2), np.round(maxObse,2),minA,maxA)
                logid.write('Keep sat/arc {0:3.0f} {1:3.0f} {2:7.2f} index {3:7.2f} min/max elev: {4:7.2f} {5:7.2f} Azims: {6:6.2f} {7:6.2f}  \n'.format( sat,iarc+1,sind,eind,np.round(minObse,2), np.round(maxObse,2), minA,maxA))

        if not nogood :
            iarc = iarc + 1
            newl = [sind, eind, int(sat), iarc,e1,e2]
            #print('indices ',sind, eind, elv[sind], elv[eind] )
            #print('indices ',sind, eind, elv[sind], elv[eind], azm[sind], azm[eind])
            tv = np.append(tv, [newl],axis=0)

    return tv 

def make_parallel_proc_lists_mjd(year, doy, year_end, doy_end, nproc):
    """
    make lists of dates for parallel processing to spawn multiple jobs

    Parameters
    ----------
    year : int
        year processing begins
    doy : int
        start day of year
    year_end : int
        year end of processing 
    doy_end  : int
        end day of year
    nproc : int
        requested number of processes to spawn

    Returns
    =======
    datelist : dict
        list of MJD 
    numproc : int
        number of datelists, thus number of processes to be used

    """
#   d = { 0: [2020, 251, 260], 1:[2020, 261, 270], 2: [2020, 271, 280], 3:[2020, 281, 290], 4:[2020,291,300] }
    # number of days for spacing ... 
    MJD1 = int(g.ydoy2mjd(year,doy))
    MJD2 = int(g.ydoy2mjd(year_end,doy_end))
    if MJD1 == MJD2:
        return [MJD1, MJD2], None

    Ndays  = math.ceil((MJD2-MJD1)/nproc) 
    #print(Ndays)
    d = {}
    i=0
    for day in range(MJD1, MJD2+1, Ndays):
        end_day = day + Ndays - 1
        if (end_day > MJD2):
            l = [day, MJD2 ]
        else:
            l = [day, end_day]
        d[i] = l
        i=i+1

    datelist = d
    numproc = i
    return datelist, numproc

def make_parallel_proc_lists(year, doy1, doy2, nproc):
    """
    make lists of dates for parallel processing to spawn multiple jobs

    Parameters
    ----------
    year : int
        year of processing
    doy1 : int
        start day of year
    doy 2 : int
        end day of year

    Returns
    -------
    datelist : dict
        list of dates formatted as year doy1 doy2
    numproc : int
        number of datelists, thus number of processes to be used

    """
#   d = { 0: [2020, 251, 260], 1:[2020, 261, 270], 2: [2020, 271, 280], 3:[2020, 281, 290], 4:[2020,291,300] }
    # number of days for spacing ... 
    Ndays  = math.ceil((doy2-doy1)/nproc) 
    #print(Ndays)
    d = {}
    i=0
    for day in range(doy1, doy2+1, Ndays):
        end_day = day+Ndays-1
        if (end_day > doy2):
            l = [year, day, doy2] 
        else:
            l = [year, day, end_day] 
        d[i] = l
        i=i+1

    datelist = d
    numproc = i
    return datelist, numproc


def open_gnssir_logfile(station,year,doy,extension):
    """
    opens a logfile when asking for screen output

    Parameters
    ----------
    station : str
        4 ch station name
    year : int
        full year
    doy : int
        day of year
    extension : str
        analysis extension name (for storage of results)
        if not set you should send empty string

    Returns
    -------
    fileid : ?
        I don't know the proper name of this - but what comes out
        when you open a file so you can keep writing to it

    """
    xdir = os.environ['REFL_CODE']
    if len(extension) == 0:
        logdir = xdir + '/logs/' + station + '/' + str(year) + '/'
    else:
        logdir = xdir + '/logs/' + station + '/' + extension + '/' + str(year) + '/'

    if not os.path.isdir(logdir):
        subprocess.call(['mkdir', '-p',logdir])
    fout = 0
    cdoy = '{:03d}'.format(doy)
#   extra file with rejected arcs

    filename = logdir + cdoy + '_gnssir.txt' 
    fileid = open(filename,'w+')
    v = str(g.version('gnssrefl'))
    fileid.write('gnssrefl version {0:s} \n'.format(v))

    return fileid, filename



# -*- coding: utf-8 -*-
import argparse
import sys

import gnssrefl.gps as g

def main():
    """
    converts year/day of year to month and day. prints to the screen

    Parameters
    ----------
    year : int
        full year 
    doy : int
        day of year

    Returns
    -------
    month : int

    day : int

    """

    parser = argparse.ArgumentParser()
    parser.add_argument("year", help="year", type=int)
    parser.add_argument("doy", help="doy", type=int)

    args = parser.parse_args()
    year = args.year
    doy = args.doy
    if (year < 1977):
        print('Year has to be after first GPS satellite launch, but you chose ', year)
        sys.exit()
    if ((doy < 1) or (doy > 366)):
        print('Illegal day of year', doy)
        sys.exit()

    year,month,day=g.ydoy2ymd(year, doy)
    mjd = g.getMJD(year,month,day,0)

    print('Month ', month, ' Day ', day, ' MJD ', mjd)


if __name__ == "__main__":
    main()
